
/**
 * Helper function to aggregate yields and uncertainties.
 *
 * If a process included in the list cannot be found in the yields objects, this
 * process is resolved to zero yield. This method uses the "base" attribute of
 * the yield objects.
 * 
 * @param yields          The object holding all yield objects from a plot. The
 *                        keys are assumed to appear in process_names.
 * @param process_names   List of names of the processes to include in the sum.
 *                        The order of the names is arbitrary.
 * @param property        Name of the property to use from the yield object,
 *                        i.e., "base" or "stat".
 * @param pre             Callback that maps bin-process values before summing
 *                        them. Should be identify for the yields, and the
 *                        square for statistical uncetainties.
 * @param post            Callback that maps bin-process values after summing
 *                        them to undo the effect of pre. Should be identify for
 *                        the yields, and the square root for statistical
 *                        uncetainties.
 * @param bin_index       If set, computes the sum of yields on the specified
 *                        bin. The index refers to the array index of "base".
 * @return                post(sum(pre(bin-process-values)))
 */
export const aggregate = ({
  yields,
  process_names,
  property,
  pre,
  post,
  bin_index
}) => {
  const getBase = (process_name) => {
    if (!yields.hasOwnProperty(process_name)) {
      return 0
    }
    const process = yields[process_name]
    if (!process.hasOwnProperty(property)) {
      return 0
    }
    const base = process[property]
    return (bin_index != null) ? base[bin_index] : base;
  }
  const agg = process_names.map(getBase).flat().map(pre).reduce((a, b) => a + b, 0)
  return post(agg)
}

/**
 * Compute the total yield from a process list.
 *
 * If a process included in the list cannot be found in the yields objects, this
 * process is resolved to zero yield. This method uses the "base" attribute of
 * the yield objects.
 * 
 * @param yields          The object holding all yield objects from a plot. The
 *                        keys are assumed to appear in process_names.
 * @param process_names   List of names of the processes to include in the sum.
 *                        The order of the names is arbitrary.
 * @param bin_index       If set, computes the sum of yields on the specified
 *                        bin. The index refers to the array index of "base".
 * @return                The sum of yields
 */
export const sumBase = (yields, process_names, bin_index=null) => (
  aggregate({
    yields,
    process_names,
    property: "base",
    pre: (x) => x,
    post: (x) => x,
    bin_index
  })
)

/**
 * Compute the total statistical uncertainty from a process list.
 *
 * If a process included in the list cannot be found in the yields objects, this
 * process is resolved to 0 uncertainty. This method uses the "stat" attribute of
 * the yield objects.
 * 
 * @param yields          The object holding all yield objects from a plot. The
 *                        keys are assumed to appear in process_names.
 * @param process_names   List of names of the processes to include in the sum.
 *                        The order of the names is arbitrary.
 * @param bin_index       If set, computes the sum of yields on the specified
 *                        bin. The index refers to the array index of "base".
 * @return                The sum of yields
 */
export const sumStat = (yields, process_names, bin_index=null) => (
  aggregate({
    yields,
    process_names,
    property: "stat",
    pre: (x) => x * x,
    post: (x) => Math.sqrt(x),
    bin_index
  })
)
