
from argparse import Namespace
from django.conf import settings
from django.contrib.auth.models import User
from django.db.models import Q
from rest_framework import generics, viewsets, status
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.decorators import api_view, permission_classes

from uhepp_vault.models import PUBLIC_LEVEL, INTERNAL_LEVEL, PUBLIC_LEVEL, \
                               Collection, Plot
from uhepp_api import serializers
from .masks import MaskRelatedColletions
from .permissions import IsOwnerOrReadOnly, IsCollectionOwnerOrReadOnly


class MaskedRelatedMixin:
    masking_backends = []

    def get_object(self):
        obj = super().get_object()
        for backend in list(self.masking_backends):
            obj = backend(obj, view=self)
        return obj

class UserViewSet(MaskedRelatedMixin, viewsets.ReadOnlyModelViewSet):
    lookup_field = 'username'
    masking_backends = [MaskRelatedColletions]

    def get_success_headers(self, data):
        username = data["username"]

        ui_url = reverse("uhepp_vault:user-detail", args=[username])
        ui_url = self.request.build_absolute_uri(ui_url)

        location = {}
        if self.action == "create":
            api_url = reverse("user-detail", args=[username])
            api_url = self.request.build_absolute_uri(api_url)
            location = {'Location': api_url}

        return {
            "Link": f'<{ui_url}>; rel="ui"',
            **location
        }
        
    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, headers=headers)
    def get_serializer_class(self):
        if self.action == 'list':
            return serializers.UserListSerializer
        return serializers.UserSerializer

    def get_queryset(self):
        if not self.request.user.is_authenticated:
            return User.objects.filter(profile__visibility__gte=PUBLIC_LEVEL)

        queryset = User.objects.filter(
            Q(profile__visibility__gte=INTERNAL_LEVEL) |
            Q(pk=self.request.user.pk)
        )
        return queryset

class CollectionViewSet(viewsets.ModelViewSet):
    permission_classes = [IsOwnerOrReadOnly]

    def get_success_headers(self, data):
        pk = data["id"]

        ui_url = reverse("uhepp_vault:collection-detail", args=[pk])
        ui_url = self.request.build_absolute_uri(ui_url)

        location = {}
        if self.action == "create":
            api_url = reverse("collection-detail", args=[pk])
            api_url = self.request.build_absolute_uri(api_url)
            location = {"Location": api_url}

        return {
            "Link": f'<{ui_url}>; rel="ui"',
            **location
        }
        
    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, headers=headers)

    def perform_create(self, serializer):
        if self.request.user.is_anonymous:
            self.permission_denied(self.request, "Permission denied")
        else:
            serializer.save(owner=self.request.user)
        
    def get_serializer_class(self):
        if self.action == 'list':
            return serializers.CollectionListSerializer
        return serializers.CollectionSerializer

    def get_queryset(self):
        if not self.request.user.is_authenticated:
            return Collection.objects.filter(visibility__gte=PUBLIC_LEVEL)

        queryset = Collection.objects.filter(
            Q(visibility__gte=INTERNAL_LEVEL) |
            Q(owner=self.request.user)
        )
        return queryset

class OwnModelViewSet:
    list_serializer_class = 2 # might be shorter
    serializer_class = 2

    # Instance operations:
    #  - permission check on instances: upon create, update, delete, get
    #    (this might include checking related objects)
    #  - Queryset filtering/masking for: list

    # Related-objects operations:
    #  - Queryset filtering for related items in output, form choice

class PlotViewSet(viewsets.ModelViewSet):
    permission_classes = [IsCollectionOwnerOrReadOnly]

    def create(self, request, *args, **kwds):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        prototype = Namespace(**serializer.validated_data)
        self.check_object_permissions(request, prototype)

        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    def get_success_headers(self, data):
        uuid = data["uuid"]

        ui_url = reverse("uhepp_vault:plot-detail", args=[uuid])
        ui_url = self.request.build_absolute_uri(ui_url)

        location = {}
        if self.action == "create":
            api_url = reverse("plot-detail", args=[uuid])
            api_url = self.request.build_absolute_uri(api_url)
            location = {"Location": api_url}

        return {
            "Link": f'<{ui_url}>; rel="ui"',
            "X-Resource-UUID": uuid,
            **location
        }
        
    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, headers=headers)

    def get_serializer(self, *args, **kwds):
        serializer = super().get_serializer(*args, **kwds)

        # Filter queryset for browsable API choices
        patching = serializer
        if hasattr(patching, "child"):
            patching = patching.child

        queryset = patching.fields["collection"].queryset
        if self.request.user.is_authenticated:
            patching.fields["collection"].queryset = queryset.filter(
                Q(owner=self.request.user) |
                Q(visibility__gte=INTERNAL_LEVEL)
            )
        else:
            patching.fields["collection"].queryset = queryset.filter(
                Q(visibility=PUBLIC_LEVEL)
            )

        return serializer

    def get_serializer_class(self):
        if self.action == 'list':
            return serializers.PlotListSerializer
        return serializers.PlotSerializer

    def get_queryset(self):
        if not self.request.user.is_authenticated:
            return Plot.objects.filter(
                collection__visibility__gte=PUBLIC_LEVEL
            )

        queryset = Plot.objects.filter(
            Q(collection__visibility__gte=INTERNAL_LEVEL) |
            Q(collection__owner=self.request.user)
        )
        return queryset
